# encoding: utf-8
"""
OOS Python API Example
"""
import oos
from ooscore.client import Config
import json
import datetime
import ooscore.exceptions as exceptions
import traceback

ENDPOINT = 'http://oos-cn.ctyunapi.cn'
IAM_ENDPOINT = 'http://oos-cn-iam.ctyunapi.cn'
ACCESS_KEY = 'fe49633483061d5b93e3'
SECRET_KEY = 'eba77e4fcdd33dbd76b4175e7e37d3136e503afd'

BUCKET = 'test-jf'
KEY = 'test'
MULTIPART_UPLOAD_KEY = 'test-multi'
UPLOAD_FILE = 'hamlet.txt'
DOWNLOAD_FILE_PATH = './foo'

# SIGNATURE_VERSION = 's3'
SIGNATURE_VERSION = 's3v4'

# unsigned payload, use a constant string UNSIGNED-PAYLOAD
# config = Config(signature_version=SIGNATURE_VERSION)
# v4 signed payload
config = Config(signature_version=SIGNATURE_VERSION, s3={'payload_signing_enabled':True})

client = oos.client('s3', use_ssl=False, endpoint_url=ENDPOINT,
                    access_key_id=ACCESS_KEY, secret_access_key=SECRET_KEY,
                    config=config)
iam_client = oos.client('sts', use_ssl=False, endpoint_url=IAM_ENDPOINT,
                        access_key_id=ACCESS_KEY, secret_access_key=SECRET_KEY,
                        config=config)


def pretty_print(dict):
    print(json.dumps(dict, sort_keys=True, indent=4, default=str, ensure_ascii=False))


def encode_params(data):
    from ooscore.compat import urlencode
    result = []
    for k, vs in data.items():
        result.append((k, vs))
    return urlencode(result, doseq=True)


# handle error
def handle_error_example():
    print('Delete none exist Bucket')
    try:
        response = client.delete_bucket(
            Bucket='NoSuchBucket'
        )
        pretty_print(response)
    except exceptions.ClientError as e:
        print('Response code: {0}, request id {1}'
              .format(e.response['Error']['Code'],
                      e.response['ResponseMetadata']['RequestId']))


def handle_error(fn):
    def inner():
        try:
            fn()
        except exceptions.ClientError as e:
            print('Response code: {0}, request id {1}'
                  .format(e.response['Error']['Code'],
                          e.response['ResponseMetadata']['RequestId']))

    return inner


# Service: 4.1.1 List Bucket
@handle_error
def list_bucket_example():
    response = client.list_buckets()
    pretty_print(response)
    for bucket in response['Buckets']:
        print('Bucket: {0}'.format(bucket['Name']))


# Bucket: 4.1.2 Get Regions
@handle_error
def get_regions_example():
    print('Get Regions')
    response = client.get_regions()
    pretty_print(response)


# Bucket: 4.2.1 Put Bucket
# 创建一个新的 bucket
@handle_error
def put_bucket_example():
    print('Creating OOS bucket')
    response = client.create_bucket(
        ACL='public-read-write',
        Bucket=BUCKET,
        CreateBucketConfiguration={
            'MetadataLocationConstraint':
                {'Location': 'QingDao'},
            'DataLocationConstraint': {
                'Type': 'Specified',
                'LocationList': ['QingDao'],
                'ScheduleStrategy': 'Allowed'}
        }
    )
    pretty_print(response)


# Bucket: 4.2.2 Get Bucket location
@handle_error
def get_bucket_location():
    print('Get Bucket location')
    response = client.get_bucket_location(
        Bucket=BUCKET
    )
    pretty_print(response)


# Bucket: 4.2.3 Get bucket acl
@handle_error
def get_bucket_acl_example():
    print('Get Bucket acl')
    response = client.get_bucket_acl(
        Bucket=BUCKET
    )
    pretty_print(response)


# Bucket: 4.2.4 Get Bucket (List Objects)
@handle_error
def get_bucket_example():
    print('List Objects')
    response = client.list_objects(
        Bucket=BUCKET,
        Marker='test',
        MaxKeys=10,
    )
    pretty_print(response)


# Bucket: 4.2.5 Delete bucket
# 删除Bucket之前必须要清空
@handle_error
def delete_bucket_example():
    print('Delete Bucket')
    response = client.delete_bucket(
        Bucket=BUCKET
    )
    pretty_print(response)


# Bucket: 4.2.6 Put Bucket Policy
@handle_error
def put_bucket_policy_example():
    print('Put Bucket policy')
    policy = {
        "Version": "2012-10-17",
        "Id": "aaaa-bbbb-cccc-dddd",
        "Statement": [
            {
                "Effect": "Allow",
                "Sid": "1",
                "Principal": {
                    "AWS": "*"
                },
                "Action": ["s3:*"],
                "Resource": "arn:aws:s3:::test-jf/*"
            }
        ]
    }
    response = client.put_bucket_policy(
        Bucket=BUCKET,
        Policy=json.dumps(policy)
    )
    pretty_print(response)


# Bucket: 4.2.7 Get Bucket Policy
@handle_error
def get_bucket_policy_example():
    print('Get Bucket policy')
    response = client.get_bucket_policy(
        Bucket=BUCKET
    )
    pretty_print(response)
    pretty_print(json.loads(response['Policy']))


# Bucket: 4.2.8 Delete bucket Policy
@handle_error
def delete_bucket_policy_example():
    print('Delete Bucket policy')
    response = client.delete_bucket_policy(
        Bucket=BUCKET
    )
    pretty_print(response)


# Bucket: 4.2.9 Put Bucket WebSite
@handle_error
def put_bucket_website_example():
    print('Put Bucket Website')
    response = client.put_bucket_website(
        Bucket=BUCKET,
        WebsiteConfiguration={
            'ErrorDocument': {
                'Key': 'error1.html'
            },
            'IndexDocument': {
                'Suffix': 'index1.html'
            }
        }
    )
    pretty_print(response)


# Bucket: 4.2.10 Get Bucket WebSite
@handle_error
def get_bucket_website_example():
    print('Get Bucket Website')
    response = client.get_bucket_website(
        Bucket=BUCKET
    )
    pretty_print(response)


# Bucket: 4.2.11 Delete Bucket WebSite
@handle_error
def delete_bucket_website_example():
    print('Delete Bucket Website')
    response = client.delete_bucket_website(
        Bucket=BUCKET
    )
    pretty_print(response)


# Bucket: 4.2.12 List Multipart Uploads
@handle_error
def list_multipart_uploads_example():
    print('List Multipart Uploads')
    response = client.list_multipart_uploads(
        Bucket=BUCKET,
        MaxUploads=100
    )
    pretty_print(response)


# Bucket: 4.2.13 Put Bucket Logging
@handle_error
def put_bucket_logging_example():
    print('Put Bucket Logging')
    response = client.put_bucket_logging(
        Bucket=BUCKET,
        BucketLoggingStatus={
            'LoggingEnabled': {
                'TargetBucket': '{0}-logging'.format(BUCKET),
                'TargetPrefix': '{0}_log'.format(BUCKET)
            }
        }
    )
    pretty_print(response)


# Bucket: 4.2.14 Get Bucket Logging
@handle_error
def get_bucket_logging_example():
    print('Get Bucket Logging')
    response = client.get_bucket_logging(
        Bucket=BUCKET
    )
    pretty_print(response)


# Bucket: 4.2.15 Head Bucket
@handle_error
def head_bucket_example():
    print('Head Bucket')
    response = client.head_bucket(
        Bucket=BUCKET
    )
    pretty_print(response)


# Bucket: 4.2.16 Put Bucket Lifecycle
@handle_error
def put_bucket_lifecycle_example():
    print('Put Bucket Lifecycle')
    # 10天后到期，用Date方式演示python sdk中整点时间处理
    d = datetime.date.today() + datetime.timedelta(days=10)
    expire_date = datetime.datetime.combine(d, datetime.datetime.min.time())
    response = client.put_bucket_lifecycle(
        Bucket=BUCKET,
        LifecycleConfiguration={
            'Rules': [
                {
                    'Expiration': {
                        'Date': expire_date
                    },
                    'ID': 'lifecycle-123',
                    'Prefix': 'test',
                    'Status': 'Enabled'
                }
            ]
        }
    )
    pretty_print(response)


# Bucket: 4.2.17 Get Bucket Lifecycle
@handle_error
def get_bucket_lifecycle_example():
    print('Get Bucket Lifecycle')
    response = client.get_bucket_lifecycle(
        Bucket=BUCKET
    )
    pretty_print(response)


# Bucket: 4.2.18 Delete Bucket Lifecycle
@handle_error
def delete_bucket_lifecycle_example():
    print('Delete Bucket Lifecycle')
    response = client.delete_bucket_lifecycle(
        Bucket=BUCKET
    )
    pretty_print(response)


# Bucket: 4.2.19 Put Bucket accelerate
@handle_error
def put_bucket_accelerate_example():
    print('Put Bucket accelerate')
    response = client.put_bucket_accelerate(
        Bucket=BUCKET,
        AccelerateConfiguration={
            'Status': 'Enabled',
            'IPWhiteLists': ['36.111.88.0/24', '114.80.1.136']
        }
    )
    pretty_print(response)


# Bucket: 4.2.20 Get Bucket accelerate
@handle_error
def get_bucket_accelerate_example():
    print('Get Bucket accelerate')
    response = client.get_bucket_accelerate(
        Bucket=BUCKET
    )
    pretty_print(response)


# Bucket: 4.2.21 Put Bucket cors
@handle_error
def put_bucket_cors_example():
    print('Put Bucket cors')
    response = client.put_bucket_cors(
        Bucket=BUCKET,
        CORSConfiguration={
            'CORSRules': [
                {
                    'AllowedOrigins': ['http://www.example1.com'],
                    'AllowedMethods': ['PUT', 'POST', 'DELETE'],
                    'AllowedHeaders': ['*']
                },
                {
                    'AllowedOrigins': ['http://www.example2.com'],
                    'AllowedMethods': ['PUT', 'POST'],
                    'AllowedHeaders': ['*']
                }
            ]
        }
    )
    pretty_print(response)


# Bucket: 4.2.22 Get Bucket cors
@handle_error
def get_bucket_cors_example():
    print('Get Bucket cors')
    response = client.get_bucket_cors(
        Bucket=BUCKET
    )
    pretty_print(response)


# Bucket: 4.2.23 Delete Bucket cors
@handle_error
def delete_bucket_cors_example():
    print('Delete Bucket cors')
    response = client.delete_bucket_cors(
        Bucket=BUCKET
    )
    pretty_print(response)


# Object: 4.3.1 Put Object
# 上传本地文件到OOS bucket
@handle_error
def put_object_example():
    print('Uploading OOS object')
    with open(UPLOAD_FILE, 'rb') as data:
        client.put_object(
            DataLocation='type=Specified,location=ChengDu,scheduleStrategy=Allowed',
            Bucket=BUCKET,
            Key=KEY,
            Body=data,
            StorageClass='EC_2_1',  # STANDARD | REDUCED_REDUNDANCY | EC_N_M
            ContentType='application/octet-stream'
        )
    print('Done')


# Object: 4.3.2 Get Object
# 下载OOS bucket文件到本地，注意设置读取response流块大小，减轻内存负载
@handle_error
def get_object_example():
    print('Downloading OOS object')
    response = client.get_object(
        Bucket=BUCKET,
        Key=KEY
    )
    pretty_print(response)
    body = response['Body']
    with open(DOWNLOAD_FILE_PATH, 'wb') as fd:
        for chunk in iter(lambda: body.read(4096), b''):
            fd.write(chunk)
    print('Done')


# Object: 4.3.3 Delete Object
@handle_error
def delete_object_example():
    print('Delete object')
    response = client.delete_object(
        Bucket=BUCKET,
        Key=KEY
    )
    pretty_print(response)


# Object: 4.3.4 Put Object-Copy
@handle_error
def put_object_copy_example():
    print('Copy object')
    response = client.copy_object(
        Bucket=BUCKET,
        CopySource={'Bucket': BUCKET, 'Key': KEY},
        Key='{0}-Copy'.format(KEY),
        DataLocation='type=Specified,location=ChengDu,scheduleStrategy=Allowed'
    )
    pretty_print(response)


# Object: 4.3.5 Initial Multipart Upload
# Object: 4.3.6 Upload Part
# Object: 4.3.7 Complete Multipart Upload
# Object: 4.3.9 List Part
@handle_error
def multipart_upload_example():
    print('Multipart Upload')
    upload = client.create_multipart_upload(
        Bucket=BUCKET,
        Key=MULTIPART_UPLOAD_KEY,
        DataLocation='type=Specified,location=ChengDu,scheduleStrategy=Allowed'
    )
    pretty_print(upload)

    MultipartUpload = {
        'Parts': []
    }
    with open(UPLOAD_FILE, 'rb') as fd:
        num = 0
        for chunk in iter(lambda: fd.read(5 * 1024 * 1024), b''):
            num += 1
            part = client.upload_part(
                Body=chunk,
                Bucket=BUCKET,
                Key=MULTIPART_UPLOAD_KEY,
                PartNumber=num,
                UploadId=upload['UploadId']
            )
            MultipartUpload['Parts'].append({
                'PartNumber': num,
                'ETag': part['ETag']
            })
            pretty_print(part)

    print('List Parts')
    list = client.list_parts(
        Bucket=BUCKET,
        Key=MULTIPART_UPLOAD_KEY,
        MaxParts=10,
        UploadId=upload['UploadId']
    )
    pretty_print(list)

    complete = client.complete_multipart_upload(
        Bucket=BUCKET,
        Key=MULTIPART_UPLOAD_KEY,
        UploadId=upload['UploadId'],
        MultipartUpload=MultipartUpload
    )
    pretty_print(complete)
    print('Done')


@handle_error
def multipart_process(q_in, q_out, UploadId):
    _config = Config(signature_version='s3')
    _client = oos.client('s3', use_ssl=False, endpoint_url=ENDPOINT,
                         access_key_id=ACCESS_KEY, secret_access_key=SECRET_KEY,
                         config=_config)
    while True:
        q_out.put(1)
        num, chunk = q_in.get()
        if num is None:
            break
        _client.upload_part(
            Body=chunk,
            Bucket=BUCKET,
            Key=MULTIPART_UPLOAD_KEY,
            PartNumber=num,
            UploadId=UploadId
        )


def multipart_upload_process_example():
    import multiprocessing

    count = 4
    processes = []

    print('Multipart Upload')
    q_in = multiprocessing.Queue()
    q_out = multiprocessing.Queue()  # 防止爆内存
    upload = client.create_multipart_upload(
        Bucket=BUCKET,
        Key=MULTIPART_UPLOAD_KEY
    )

    for i in range(count):
        p = multiprocessing.Process(target=multipart_process, args=(q_in, q_out, upload['UploadId']))
        p.start()
        processes.append(p)

    with open(UPLOAD_FILE, 'rb') as fd:
        num = 0
        for chunk in iter(lambda: fd.read(5 * 1024 * 1024), b''):
            q_out.get()
            num += 1
            q_in.put((num, chunk))

    for i in range(count):
        q_in.put((None, None))  # sentinel

    for p in processes:
        p.join()

    complete = client.complete_multipart_upload(
        Bucket=BUCKET,
        Key=MULTIPART_UPLOAD_KEY,
        UploadId=upload['UploadId']
    )
    pretty_print(complete)
    print('Done')


# Object: 4.3.8 Abort Multipart Upload
@handle_error
def abort_multipart_example():
    print('Abort Multipart')
    upload = client.create_multipart_upload(
        Bucket=BUCKET,
        Key=MULTIPART_UPLOAD_KEY,
        DataLocation='type=Specified,location=ChengDu,scheduleStrategy=Allowed'
    )
    pretty_print(upload)

    response = client.abort_multipart_upload(
        Bucket=BUCKET,
        Key=MULTIPART_UPLOAD_KEY,
        UploadId=upload['UploadId']
    )
    pretty_print(response)


# Object: 4.3.10 Copy Part
@handle_error
def copy_part_example():
    print('Copy Part')
    upload = client.create_multipart_upload(
        Bucket=BUCKET,
        Key=MULTIPART_UPLOAD_KEY
    )
    pretty_print(upload)

    part = client.upload_part_copy(
        Bucket=BUCKET,
        CopySource={'Bucket': BUCKET, 'Key': KEY},
        # CopySourceRange='bytes=0-9',
        Key=MULTIPART_UPLOAD_KEY,
        PartNumber=1,
        UploadId=upload['UploadId']
    )
    pretty_print(part)

    MultipartUpload = {
        'Parts': [
            {'PartNumber': 1,
             'ETag': part['CopyPartResult']['ETag']}
        ]
    }

    complete = client.complete_multipart_upload(
        Bucket=BUCKET,
        Key=MULTIPART_UPLOAD_KEY,
        UploadId=upload['UploadId'],
        MultipartUpload=MultipartUpload
    )
    pretty_print(complete)
    print('Done')


# Object: 4.3.12 Delete Multiple Objects
@handle_error
def delete_multiple_example():
    print('Delete Multiple Objects')
    response = client.delete_objects(
        Bucket=BUCKET,
        Delete={
            'Objects': [
                {
                    'Key': 'test'
                },
                {
                    'Key': 'test-Copy'
                }
            ]
        }
    )
    pretty_print(response)


# Object 4.3.16 Generate Shared link
@handle_error
def generate_shared_link_example():
    print('Generate Shared link')
    shared_link = client.generate_presigned_url(
        ClientMethod='get_object',
        Params={
            'Bucket': BUCKET,
            'Key': KEY
        },
        #ExpiresIn=3600      # 过期时间
    )
    print(shared_link)


# Object 4.3.17 Head Object
@handle_error
def head_object_example():
    print('Head Object')
    response = client.head_object(
        Bucket=BUCKET,
        Key=KEY,
        IfMatch='3adbbad1791fbae3ec908894c4963871'
    )
    pretty_print(response)


# AccessKey: 4.4.1 Create Access Key
@handle_error
def create_access_key_example():
    print('Create access key')
    data = {
        "Action": "CreateAccessKey"
    }
    body = encode_params(data)
    response = iam_client.create_access_key(
        Body=body
    )
    pretty_print(response)


# AccessKey: 4.4.2 Delete Access Key
@handle_error
def delete_access_key_example():
    print('Delete access key')
    data = {
        "Action": "DeleteAccessKey",
        "AccessKeyId": "cab3d9f35de9051a66d9"
    }
    body = encode_params(data)
    response = iam_client.delete_access_key(
        Body=body
    )
    pretty_print(response)


# AccessKey: 4.4.3 Update Access Key
@handle_error
def update_access_key_example():
    print('Update access key')
    data = {
        "Action": "UpdateAccessKey",
        "AccessKeyId": "c87e8256ba64b8313761",
        "Status": "Inactive",  # Active or Inactive
        "IsPrimary": False
    }
    body = encode_params(data)
    response = iam_client.update_access_key(
        Body=body
    )
    pretty_print(response)


# AccessKey: 4.4.4 List Access Key
@handle_error
def list_access_key_example():
    print('List access key')
    data = {
        'Action': 'ListAccessKey',
        'MaxItems': 10
    }
    body = encode_params(data)
    response = iam_client.list_access_key(
        Body=body
    )
    pretty_print(response)


if __name__ == '__main__':
    try:
        list_access_key_example()
    except Exception as ex:
        print(traceback.format_exc())
        print(ex)
